ScheduleBuilt

Struct ScheduleBuilt 

pub struct ScheduleBuilt {
    pub label: InternedScheduleLabel,
    pub build_metadata: ScheduleBuildMetadata,
}
An event triggered when a schedule is successfully built.

Note: When this event is triggered, the corresponding Schedule is not present in the world. So, observers will need to cache whatever data they need from this and access it later once the schedule is not running.

Fields

label: InternedScheduleLabel

The schedule that was built.

build_metadata: ScheduleBuildMetadata

The metadata for the build process of this schedule.