pub struct ScheduleBuilt {
pub label: InternedScheduleLabel,
pub build_metadata: ScheduleBuildMetadata,
}Expand description
An event triggered when a schedule is successfully built.
Note: When this event is triggered, the corresponding Schedule is not present in the world.
So, observers will need to cache whatever data they need from this and access it later once the
schedule is not running.
Fields
label: InternedScheduleLabelThe schedule that was built.
build_metadata: ScheduleBuildMetadataThe metadata for the build process of this schedule.